/*
 * util.C: Miscellaneous functions.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 */

static char const rcsid[] = "$Id: numeric.C,v 1.1 2007/09/22 14:27:26 pippijn Exp $";

#include <cstdio>
#include <stdint.h>

#include <util/numeric.h>

int
itoa (uint32_t n, char *buf, int buflen)
{
  uint32_t i;
  uint32_t out = 0;

  if (n < 10)
    {
      *buf = n + '0';
      return 1;
    }
  
  if (buflen > 0)
    {
      buflen -= 11; // maximum number of digits
      for (i = uint32_t (1e9); i; i = (i * 0x1999999aUL) >> 32) // divide by 10
        {
          uint32_t div = n / i;
          n %= i; // the compiler reuses the remainder produced by dividing
          if (div || out || i == 1)
            {
              out++;
              if (buflen >= 0)
                *buf++ = '0' + div;
            }
          buflen++;
        }
      *buf = 0;
    }
  return out;
}

int
itoa (uint64_t n, char *buf, int buflen)
{
  if (n <= uint32_t (-1))
    return itoa (uint32_t (n), buf, buflen);
  else if (n < uint64_t (1e18))
    {
      uint32_t div = n / uint32_t (1e9);
      uint32_t rem = n % uint32_t (1e9);
      
      uint32_t len = itoa (div, buf, buflen);

      for (uint32_t z = uint32_t (1e8); z > 1; z = (z * 0x1999999aUL) >> 32)
        {
          *(buf + len) = '0';
          len += (rem < z);
        }

      return itoa (rem, buf + len, buflen - len);
    }
  else if (n < uint64_t (1e19))
    {
      uint32_t div = n / uint64_t (1e18);
      uint64_t rem = n % uint64_t (1e18);

      *buf++ = '0' + div;
      
      return itoa (rem, buf, buflen - 1);
    }
  else
    {
      uint32_t div = n / uint64_t (1e19);
      uint64_t rem = n % uint64_t (1e19);

      *buf++ = '0' + div;
      
      return itoa (rem, buf, buflen);
    }
}

