#include "tst-main.C"

#define BLOCK_SIZE 100000

int
exec_tests ()
{
#if defined(HAVE_ALTIVEC)
  static vector unsigned int array1[BLOCK_SIZE / 4];
  static vector unsigned int array2[10000 / 4];
#elif defined(HAVE_SSE2)
  static __m128i array1[BLOCK_SIZE / 4];
  static __m128i array2[10000 / 4];
#else
  static uint64_t array1[BLOCK_SIZE / 4][2];
  static uint64_t array2[10000 / 4][2];
#endif
  
  int i;
  uint32_t *array32 = reinterpret_cast<uint32_t *> (array1);
  uint32_t *array32_2 = reinterpret_cast<uint32_t *> (array2);
  uint32_t ini[4] = { 0x1234, 0x5678, 0x9abc, 0xdef0 };
  uint32_t r32;
  
  assert (get_min_array_size32 () < 10000);

  /* 32 bit generation */
  init_gen_rand (1234);
  fill_array32 (array32, 10000);
  fill_array32 (array32_2, 10000);
  init_gen_rand (1234);
  for (i = 0; i < 10000; i++)
    {
      r32 = gen_rand32 ();
      assert (r32 == array32[i]);
    }
  for (i = 0; i < 700; i++)
    {
      r32 = gen_rand32 ();
      assert (r32 == array32_2[i]);
    }
  init_by_array (ini, 4);
  fill_array32 (array32, 10000);
  fill_array32 (array32_2, 10000);
  init_by_array (ini, 4);
  for (i = 0; i < 10000; i++)
    {
      r32 = gen_rand32 ();
      assert (r32 == array32[i]);
    }
  for (i = 0; i < 700; i++)
    {
      r32 = gen_rand32 ();
      assert (r32 == array32_2[i]);
    }

  uint64_t *array64;
  uint64_t *array64_2;
  uint64_t r;
  uint32_t ini2[] = { 5, 4, 3, 2, 1 };

  array64 = reinterpret_cast<uint64_t *> (array1);
  array64_2 = reinterpret_cast<uint64_t *> (array2);
  assert (get_min_array_size64 () < 5000 && "array size too small!");
  /* 64 bit generation */
  init_gen_rand (4321);
  fill_array64 (array64, 5000);
  fill_array64 (array64_2, 5000);
  init_gen_rand (4321);
  for (i = 0; i < 5000; i++)
    {
      r = gen_rand64 ();
      assert (r == array64[i]);
    }
  for (i = 0; i < 700; i++)
    {
      r = gen_rand64 ();
      assert (r == array64_2[i]);
    }
  /* 64 bit generation */
  init_by_array (ini2, 5);
  fill_array64 (array64, 5000);
  fill_array64 (array64_2, 5000);
  init_by_array (ini2, 5);
  for (i = 0; i < 5000; i++)
    {
      r = gen_rand64 ();
      assert (r == array64[i]);
    }
  for (i = 0; i < 700; i++)
    {
      r = gen_rand64 ();
      assert (r == array64_2[i]);
    }

  return 0;
}
